<?php
/********
Plugin Name: Category Order
Plugin URI: http://www.coppit.org/code/
Description: Sorts the sidebar category list according to a user-defined 'Category Order' option in the 'Manage' menu 
Version: 1.9.7
Author: David Coppit
Author URI: http://www.coppit.org/
*********/

//==========================================================================

/*
Notes on options, for developers:

- Category_Order_Put_Post_Count_Inside_Link: A Boolean that indicates whether
  the number of posts "(8)" should be inside the link for the category
- Category_Order: A serialized data structure that represents the order,
  nesting, and spacing of categories.

This plugin automatically creates defaults for these options. For the first,
the lack of an option is equivalent to "false". For the second, the plugin
will query the database of categories, adding them in alphabetical order to
the data structure at the top level.

The plugin always checks the database, removing or adding categories as
necessary.
*/

/*
Notes on the category order data structure, for developers:

If this is the category structure that the user wants:

  1
  - 1.1
  2
  - 2.1
  - 2.2
  3
  [A space]
  4
  - 4.1

The associated data structure is identical to the one produced by the
following code:

  $category_order = array();
  $category_order[0] = array('1',0);
  $category_order[1] = array('1.1',1);
  $category_order[2] = array('2',0);
  $category_order[3] = array('2.1',1);
  $category_order[4] = array('2.2',1);
  $category_order[5] = array('3',0);
  $category_order[6] = array('',0);
  $category_order[7] = array('4',0);
  $category_order[8] = array('4.1',1);

The category_order hash maps parents to their ordered children. '1', '2', etc.
are the category slugs. 
*/

//==========================================================================

include_once('category-order-management.php');
include_once('category-order-shared.php');

$debug_category_order = false;

class category_order {

// We assume that $original_html is valid HTML, with <li> or <br /> for each
// list item. We ignore any sublists, overriding them with ours.
function sort_categories($original_html) {
//if (preg_match('/</i',$original_html)) print "vvvvvvvvvv\n$original_html\n^^^^^^^^^^\n";

	if (get_bloginfo('version') >= 2.1 &&
			preg_match('/(<li class="categories">.*?<ul>\s*)(.*)(<\/ul.*)/si',$original_html,$matches))
	{
		$prefix = $matches[1];
		$processed_html = $matches[2];
		$suffix = $matches[3];
	} else {
		$prefix = '';
		$processed_html = $original_html;
		$suffix = '';
	}

	$slug_to_item = category_order::extract_items($processed_html);
//category_order::print_html_data($slug_to_item);
	if (is_null($slug_to_item))
		return $original_html;

	$new_html = category_order::build_list_html($slug_to_item, $prefix, $suffix);

	return $new_html;
}

// --------------------------------------------------------------------

// Parses the HTML given to us by Wordpress, extracting the
// link/post-count/etc from it. Returns a hash mapping the category slug to
// this information.
function extract_items($original_html) {
/*
$original_html= '
<li style="margin-left:1em;"><a href="http://www.coppit.org/blog/archives/category/1/" title="View all
posts filed under [Other]">[Other]</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/christopher/"
title="View all posts filed under Christopher">Christopher</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/dorothy/"
title="View all posts filed under Dorothy">Dorothy</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/this-blog/"
title="View all posts filed under This Blog">This Blog</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/david/" title="View
all posts filed under David">David</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/cool-tech/"
title="View all posts filed under Cool Tech">Cool Tech</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/family/" title="View
all posts filed under Family">Family</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/friends/"
title="View all posts filed under Friends">Friends</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/trips/" title="View
all posts filed under Trips">Trips</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/software/"
title="View all posts filed under Software">Software</a>
</li>
<li><a href="http://www.coppit.org/blog/archives/category/deafness-and-cis/"
title="View all posts filed under Deafness &#038; CIs">Deafness &#038; CIs</a>
</li>
';
*/

	// Remove all <h2>, <ul>, </ul>, and </li>
	$processed_html = preg_replace('/(<\s*(ul|h\d)[^>]*>|<\/\s*ul\s*>)/i',
		'',$original_html);

	// Split up the list, then remove all empty items and clean up the HTML a
	// bit. Empty items can occur if <ul> is removed for a sublist, leaving
	// <li>\s*<li>. It can also occur at the start of the list after the split.
	// Also remove any whitespace before or after.
	if (preg_match('/<\s*li/i',$processed_html)) {
		$items = preg_split('/<\s*li\b/i', $processed_html);

		for ($i=0; $i < count($items); $i++) {
			$items[$i] = '<li' . $items[$i];

			$items[$i] = preg_replace('/\n/',' ',$items[$i]);
			$items[$i] = preg_replace('/^\s*(.*?)\s*$/s','$1',$items[$i]);

			if (preg_match('/^<\s*li$/',$items[$i])) {
				array_splice($items,$i,1);
				$i--;
			}
		}
	} elseif (preg_match('/<\s*br\s*\/>/i',$processed_html)) {
		$items = preg_split('/<\s*br\s*\/>/i', $processed_html);

		for ($i=0; $i < count($items); $i++) {
			$items[$i] = $items[$i] . '<br/>';

			$items[$i] = preg_replace('/\n/',' ',$items[$i]);
			$items[$i] = preg_replace('/^\s*(.*?)\s*$/s','$1',$items[$i]);

			if (preg_match('/^<\s*br[^>]*>$/',$items[$i])) {
				array_splice($items,$i,1);
				$i--;
			}
		}
	} else {
		return null;
	}

	// Move the count inside the link
	if (get_option('Category_Order_Put_Post_Count_Inside_Link')) {
		for ($i=0; $i < count($items); $i++)
			$items[$i] =
				preg_replace('/\s*<\/a>\s*(\(\d+\))/i',' $1</a>',$items[$i]);
	}

	global $wp_rewrite;
	$category_permastruct = $wp_rewrite->get_category_permastruct();

	if (empty($category_permastruct)) {
		$using_category_id = true;
		$category_pattern =
			'/' . preg_quote(get_settings('home') . '/?cat=', '/') . '(\d+)/';
	} else {
		$using_category_id = false;
		$category_pattern = '/' . preg_quote(get_settings('home'), '/') .
			'.*?' . preg_quote($category_permastruct, '/') . '/';
		$category_pattern =
			preg_replace('/%category%/', '(?:[^\/"]+\/)*([^\/"]+)', $category_pattern);
	}

//$category_pattern = '/http\:\/\/www\.coppit\.org\/blog.*?\/category\/(?:[^\/"]+\/)*([^\/"]+)/';

	// Make a hash for easy lookup
	$slug_to_item = array();
	foreach ($items as $item) {
		$matched = preg_match($category_pattern,$item,$matches);

		global $debug_category_order;

		if (!$matched || $debug_category_order) {
			print '<div style="position:absolute;top:0;left:0;background-color:yellow;width:body.clientWidth;color:black;">
			';
			if (!$matched)
				print '
<div style="position:absolute;top:0;left:0;background-color:yellow;width:body.clientWidth;color:black;">
<p>Category Order could not understand your category HTML. Please do the
following:</p>
<ul style="color:black;list-style-type:disc;margin-left:1em;">
<li> Disable all plugins except for category order to see if some other plugin
is causing the problem. If the problem goes away, re-enable each plugin until
you find the incompatible one.
<li> Try switching to a different theme, like the WordPress default theme.
</ul>
<p>Once you have tried the above steps, email <a
href="mailto:david@coppit.rg">david@coppit.org</a> with the results of these
debugging steps. Also include the following information:</p>
			';
			print "<p>Original HTML:<br>\n";
			category_order::print_html_data($original_html);
			print '</p><p>Category pattern:<br>';
			category_order::print_html_data($category_pattern);
			print '</p><p>Items:<br>';
			category_order::print_html_data($items);
			print "</p></div>\n";
			return null;
		}

		if ($using_category_id) {
			$category = get_category($matches[1]);
			$slug = $category->category_nicename;
		} else
			$slug = $matches[1];

		$slug_to_item[$slug] = $item;
	}

	return $slug_to_item;
}

// --------------------------------------------------------------------

function build_list_html($slug_to_item, $prefix, $suffix) {
	$category_order = category_order_shared::get_category_order();

	unset($last_nonspacer_index);
	$number_of_ending_spacers = 0;

	for ($i=count($category_order)-1; $i >= 0; $i--) {
        # In case the category was filtered out in wp_list_cats
//print $category_order[$i][0] . "::" . $slug_to_item[$category_order[$i][0]] .  "<br>\n";
		if ($slug_to_item[$category_order[$i][0]] == '')
			continue;

		if ($category_order[$i][0] == '')
			$number_of_ending_spacers++;
		else {
			$last_nonspacer_index = $i;
			break;
		}
	}

	if (preg_match('/<\s*li/i',$slug_to_item[$category_order[0][0]])) {
		$html = '';

		$number_of_leading_spacers = 0;

		for ($i=0; $i < count($category_order); $i++) {
			if ($category_order[$i][0] == '') {
				$number_of_leading_spacers++;
				continue;
			}

			if ($slug_to_item[$category_order[$i][0]] == '')
				continue;

			if ($number_of_leading_spacers == 0 &&
					$category_order[$i][1] == 0 &&
					(!isset($last_nonspacer_index) ||
					 $i != $last_nonspacer_index ||
					 $number_of_ending_spacers == 0)) {
				$html .= $slug_to_item[$category_order[$i][0]] . "\n";
				continue;
			}

			$slug_to_item[$category_order[$i][0]] =
				preg_replace("/<\s*li\s*/i", "", $slug_to_item[$category_order[$i][0]] );

			$existing_style = '';

			if (preg_match('/[^>]*(style\s*=\s*[\'"](.*?)[\'"])/i',
					$slug_to_item[$category_order[$i][0]], $matches)) {
				$existing_style = $matches[2];

				$slug_to_item[$category_order[$i][0]] =
					preg_replace('/' . $matches[1] . '/', "",
					$slug_to_item[$category_order[$i][0]] );
			}

			$html .= "<li style=\"$existing_style";

			if ($number_of_leading_spacers != 0)
				$html .= "margin-top:${number_of_leading_spacers}em;";

			if ($category_order[$i][1] != 0)
				$html .= 'margin-left:' . $category_order[$i][1] . 'em;';

			if (isset($last_nonspacer_index) && $i == $last_nonspacer_index &&
					$number_of_ending_spacers != 0)
				$html .= "margin-bottom:${number_of_ending_spacers}em;";

			$html .= '"' . $slug_to_item[$category_order[$i][0]] . "\n";

			$number_of_leading_spacers = 0;
		}
	}
	else
	{
		$number_of_leading_spacers = 0;
		$html = "<div style=\"margin-top:0em;margin-bottom:0em;\">\n";

		for ($i=0; $i < count($category_order); $i++) {
			if ($slug_to_item[$category_order[$i][0]] == '')
				continue;

			if ($category_order[$i][0] == '') {
				$number_of_leading_spacers++;
				continue;
			}

			if ($number_of_leading_spacers == 0 &&
					$category_order[$i][1] == 0 &&
					(!isset($last_nonspacer_index) ||
					 $i != $last_nonspacer_index ||
					 $number_of_ending_spacers == 0)) {
				$html .= "<div> " . $slug_to_item[$category_order[$i][0]] . " </div>\n";
				continue;
			}

			$slug_to_item[$category_order[$i][0]] =
				preg_replace("/<\s*br[^>]*>\s*/i", "", $slug_to_item[$category_order[$i][0]] );


			$html .= '<div style="';

			if ($number_of_leading_spacers != 0)
				$html .= "margin-top:${number_of_leading_spacers}em;";

			if ($category_order[$i][1] != 0)
				$html .= 'margin-left:' . $category_order[$i][1] . 'em;';

			if (isset($last_nonspacer_index) && $i == $last_nonspacer_index &&
					$number_of_ending_spacers != 0)
				$html .= "margin-bottom:${number_of_ending_spacers}em;";

			$html .= '"> ' . $slug_to_item[$category_order[$i][0]] . " </div>\n";

			$number_of_leading_spacers = 0;
		}

		$html .= "</div>\n";
	}

	return $prefix.$html.$suffix;
}

// --------------------------------------------------------------------

// Used during development to dump a data structure to html
function print_html_data($data) {
	$string = htmlspecialchars(print_r($data,1));
	$string = preg_replace("/\n/", "<br>\n", $string);
	$string = preg_replace("/ /", "&nbsp;", $string);

	print $string;
}

}

add_action('wp_list_categories', array('category_order','sort_categories'));

// For versions prior to 2.1
add_action('list_cats', array('category_order','sort_categories'));

?>
